/**
 * @file
 *
 * @brief
 *
 * @copyright BSD License (see LICENSE.md or https://www.libelektra.org)
 */

#include <kdb.h>
#include <kdbconfig.h>
#include <kdbmacros.h>
#include <kdbplugin.h>

#include <string.h>

#include <kdberrors.h>

int elektraConstantsGet (Plugin * handle ELEKTRA_UNUSED, KeySet * returned ELEKTRA_UNUSED, Key * parentKey ELEKTRA_UNUSED);
int elektraConstantsSet (Plugin * handle ELEKTRA_UNUSED, KeySet * returned ELEKTRA_UNUSED, Key * parentKey ELEKTRA_UNUSED);

static void elektraAddKeyValue (KeySet * ks, Key * parentKey, const char * key, const char * value)
{
	Key * k = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (k, key);
	keySetString (k, value);
	ksAppendKey (ks, k);
}

static void elektraAddConstants (KeySet * ks, Key * parentKey)
{
	Key * key = keyDup (parentKey, KEY_CP_ALL);
	ksAppendKey (ks, key);

	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "version");
	keySetString (key, "Duplicate of version info also found below system:/elektra/version");
	ksAppendKey (ks, key);


	key = keyDup (key, KEY_CP_ALL);
	keyAddName (key, "version/KDB_VERSION");
	keySetString (key, "0.11.0");
	ksAppendKey (ks, key);

	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "version/KDB_VERSION_MAJOR");
	keySetString (key, "0");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "version/KDB_VERSION_MINOR");
	keySetString (key, "11");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "version/KDB_VERSION_PATCH");
	keySetString (key, "0");
	ksAppendKey (ks, key);


	elektraAddKeyValue (ks, parentKey, "version/SO_VERSION", "5");
	elektraAddKeyValue (ks, parentKey, "version/SO_VERSION_TOOLS", "2");
	elektraAddKeyValue (ks, parentKey, "version/SO_VERSION_GETENV", "0");


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "cmake");
	keySetString (key, "All cmake variables as defined in the file cmake/ElektraCache.cmake");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "cmake/PLUGINS");
	keySetString (key, "ansible;augeas;backend;backend_odbc;base64;blacklist;blockresolver;c;ccode;conditionals;constants;counter;cpptemplate;crypto;csvstorage;curlget;date;dbus;dbusrecv;desktop;directoryvalue;doc;dpkg;dump;email;error;fcrypt;file;filecheck;fstab;gitresolver;glob;gopts;gpgme;hexcode;hexnumber;hosts;iconv;internalnotification;ipaddr;iterate;jni;journald;kconfig;keytometa;length;line;lineendings;logchange;lua;macaddr;mathcheck;mini;missing;modules;mozprefs;network;ni;noresolver;passwd;path;process;profile;python;quickdump;range;recorder;reference;rename;resolver_fm_b_b;resolver_fm_hb_b;resolver_fm_hp_b;resolver_fm_hpu_b;resolver_fm_pb_b;resolver_fm_ub_x;resolver_fm_uhb_xb;resolver_fm_xb_x;resolver_fm_xhp_x;resolver_fm_xp_x;rgbcolor;ruby;shell;simpleini;spec;specload;sync;syslog;template;timeofday;toml;tracer;type;uname;unit;validation;version;wresolver;xerces;xfconf;xmltool;yajl;yamlcpp;zeromqrecv;zeromqsend");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "cmake/TOOLS");
	keySetString (key, "kdb;race;qt-gui");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "cmake/KDB_DB_SYSTEM");
	keySetString (key, "/etc/kdb");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "cmake/KDB_DB_HOME");
	keySetString (key, "/home");
	ksAppendKey (ks, key);

	elektraAddKeyValue (ks, parentKey, "cmake/KDB_DB_SPEC", "/usr/share/elektra/specification");
	elektraAddKeyValue (ks, parentKey, "cmake/KDB_DB_DIR", ".dir");
	elektraAddKeyValue (ks, parentKey, "cmake/KDB_DB_FILE", "default.ecf");
	elektraAddKeyValue (ks, parentKey, "cmake/KDB_DB_INIT", "elektra.ecf");
	elektraAddKeyValue (ks, parentKey, "cmake/KDB_DEFAULT_RESOLVER", "resolver_fm_hpu_b");
	elektraAddKeyValue (ks, parentKey, "cmake/KDB_DEFAULT_STORAGE", "dump");

	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "cmake/KDB_DB_USER");
	keySetString (key, ".config");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "cmake/GTEST_ROOT");
	keySetString (key, "");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "cmake/BUILD_SHARED");
	keySetString (key, "ON");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "cmake/BUILD_FULL");
	keySetString (key, "OFF");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "cmake/BUILD_STATIC");
	keySetString (key, "ON");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "cmake/BINDINGS");
	keySetString (key, "");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "cmake/TARGET_INCLUDE_FOLDER");
	keySetString (key, "elektra");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "cmake/TARGET_CMAKE_FOLDER");
	keySetString (key, "/usr/share/cmake/Modules");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "cmake/TARGET_PLUGIN_FOLDER");
	keySetString (key, "elektra");
	ksAppendKey (ks, key);

	elektraAddKeyValue (ks, parentKey, "cmake/ENABLE_DEBUG", "OFF");
	elektraAddKeyValue (ks, parentKey, "cmake/ENABLE_LOGGER", "OFF");
	elektraAddKeyValue (ks, parentKey, "cmake/ENABLE_ASAN", "OFF");

	elektraAddKeyValue (ks, parentKey, "cmake/BUILTIN_PLUGIN_FOLDER", BUILTIN_PLUGIN_FOLDER);
	elektraAddKeyValue (ks, parentKey, "cmake/BUILTIN_DATA_FOLDER", BUILTIN_DATA_FOLDER);
	elektraAddKeyValue (ks, parentKey, "cmake/BUILTIN_EXEC_FOLDER", BUILTIN_EXEC_FOLDER);

	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "cmake/TARGET_PKGCONFIG_FOLDER");
	keySetString (key, "pkgconfig");
	ksAppendKey (ks, key);

	elektraAddKeyValue (ks, parentKey, "cmake/TARGET_DOCUMENTATION_TEXT_FOLDER", "share/doc/elektra");
	elektraAddKeyValue (ks, parentKey, "cmake/TARGET_DOCUMENTATION_HTML_FOLDER", "share/doc/elektra-api/html");
	elektraAddKeyValue (ks, parentKey, "cmake/TARGET_DOCUMENTATION_MAN_FOLDER", "share/man/man3");
	elektraAddKeyValue (ks, parentKey, "cmake/TARGET_DOCUMENTATION_LATEX_FOLDER", "share/doc/elektra-api/latex");

	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "cmake/TARGET_TOOL_EXEC_FOLDER");
	keySetString (key, "lib/elektra/tool_exec");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "cmake/TARGET_TOOL_DATA_FOLDER");
	keySetString (key, "share/elektra/tool_data");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "cmake/TARGET_TEST_DATA_FOLDER");
	keySetString (key, "share/elektra/test_data");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "cmake/TARGET_TEMPLATE_FOLDER");
	keySetString (key, "share/elektra/templates");
	ksAppendKey (ks, key);

	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "cmake/LIB_SUFFIX");
	keySetString (key, "");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "cmake/CMAKE_INSTALL_PREFIX");
	keySetString (key, "/usr");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "compiler");
	keySetString (key, "Flags defined for compilers as defined in ElektraCompiling.cmake");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "compiler/c_flags");
	keySetString (key, "-O2 -fwrapv -pipe -gdwarf-4 -fno-debug-types-section -fvar-tracking-assignments -g2 -Wp,-D_FORTIFY_SOURCE=2 -fstack-protector-strong --param=ssp-buffer-size=4 -Werror=trampolines -fomit-frame-pointer -march=i686 -fasynchronous-unwind-tables -mtune=pentium4 -gdwarf-4 -fno-debug-types-section -fvar-tracking-assignments -g2 -Wformat -Werror=format-security -std=gnu99  -Wno-deprecated-declarations  -Wstrict-prototypes  -Wno-long-long -Wpedantic -Wno-variadic-macros -Wall -Wextra -Wno-overlength-strings -Wsign-compare -Wfloat-equal -Wformat -Wformat-security -Wshadow -Wcomments -Wtrigraphs -Wundef -Wuninitialized -Winit-self -Wmaybe-uninitialized -Wsign-compare -Wfloat-equal");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "compiler/id");
	keySetString (key, "GNU");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "compiler/coverage");
	keySetString (key, "OFF");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "compiler/cxx_flags");
	keySetString (key, "-O2 -fwrapv -pipe -gdwarf-4 -fno-debug-types-section -fvar-tracking-assignments -g2 -Wp,-D_FORTIFY_SOURCE=2 -fstack-protector-strong --param=ssp-buffer-size=4 -Werror=trampolines -fomit-frame-pointer -march=i686 -fasynchronous-unwind-tables -mtune=pentium4 -gdwarf-4 -fno-debug-types-section -fvar-tracking-assignments -g2 -Wformat -Werror=format-security -std=c++11  -Wno-deprecated-declarations  -Wold-style-cast -Wstrict-null-sentinel -D_GLIBCXX_USE_NANOSLEEP -Wno-missing-field-initializers -Woverloaded-virtual  -Wsign-promo  -Wno-long-long -Wpedantic -Wno-variadic-macros -Wall -Wextra -Wno-overlength-strings -Wsign-compare -Wfloat-equal -Wformat -Wformat-security -Wshadow -Wcomments -Wtrigraphs -Wundef -Wuninitialized -Winit-self -Wmaybe-uninitialized");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "compiler/pic_flags");
	keySetString (key, "-fPIC");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "compiler/static_flags");
	keySetString (key, "");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "macros");
	keySetString (key, "Define the macros from kdbos.h and kdbconfig.h");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "macros/KDB_MAX_PATH_LENGTH");
	keySetString (key, ELEKTRA_STRINGIFY (KDB_MAX_PATH_LENGTH));
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "macros/KDB_FILE_MODE");
	keySetString (key, ELEKTRA_STRINGIFY (KDB_FILE_MODE));
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "macros/KDB_DIR_MODE");
	keySetString (key, ELEKTRA_STRINGIFY (KDB_DIR_MODE));
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "macros/KDB_PATH_SEPARATOR");
	keySetString (key, ELEKTRA_STRINGIFY (KDB_PATH_SEPARATOR));
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "macros/KDB_PATH_ESCAPE");
	keySetString (key, ELEKTRA_STRINGIFY (KDB_PATH_ESCAPE));
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "package");
	keySetString (key, "Infos about the package as it is installed");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "package/prefix");
	keySetString (key, "/usr");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "package/libdir");
	keySetString (key, "/usr/lib");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "package/includedir");
	keySetString (key, "/usr/include/elektra");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "package/plugindir");
	keySetString (key, "/usr/lib/elektra");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "package/tool_execdir");
	keySetString (key, "/usr/lib/elektra/tool_exec");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "package/templatedir");
	keySetString (key, "/usr/share/elektra/templates");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "package/libs");
	keySetString (key, "-L/usr/lib -lelektra");
	ksAppendKey (ks, key);


	key = keyDup (parentKey, KEY_CP_ALL);
	keyAddName (key, "package/cflags");
	keySetString (key, "-I/usr/include/elektra");
	ksAppendKey (ks, key);
}

// TODO [new_backend]: this plugin should be standalone backend like version
int elektraConstantsGet (Plugin * handle ELEKTRA_UNUSED, KeySet * ks, Key * parentKey)
{
	if (!strcmp (keyName (parentKey), "system:/elektra/modules/constants"))
	{
		ksAppendKey (ks,
			     keyNew ("system:/elektra/modules/constants", KEY_VALUE, "constants plugin waits for your orders", KEY_END));
		ksAppendKey (ks, keyNew ("system:/elektra/modules/constants/exports", KEY_END));
		ksAppendKey (ks, keyNew ("system:/elektra/modules/constants/exports/get", KEY_FUNC, elektraConstantsGet, KEY_END));
		ksAppendKey (ks, keyNew ("system:/elektra/modules/constants/exports/set", KEY_FUNC, elektraConstantsSet, KEY_END));
		KeySet * contract =
			ksNew (30,
#include "readme_constants.c"
			       keyNew ("system:/elektra/modules/constants/infos/version", KEY_VALUE, PLUGINVERSION, KEY_END), KS_END);
		ksAppend (ks, contract);
		ksDel (contract);
		return 1;
	}

	elektraAddConstants (ks, parentKey);

	return 1; /* success */
}

int elektraConstantsSet (Plugin * handle ELEKTRA_UNUSED, KeySet * returned, Key * parentKey)
{
	KeySet * info = ksNew (0, KS_END);
	elektraAddConstants (info, parentKey);
	ELEKTRA_SET_ERROR_READ_ONLY (info, returned, parentKey);
	return 0;
}

Plugin * ELEKTRA_PLUGIN_EXPORT
{
	// clang-format off
	return elektraPluginExport("constants",
		ELEKTRA_PLUGIN_GET,	&elektraConstantsGet,
		ELEKTRA_PLUGIN_SET,	&elektraConstantsSet,
		ELEKTRA_PLUGIN_END);
}
